#property copyright "Copyright  2008, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_chart_window
#property indicator_buffers 8
#property indicator_color1 Lime
#property indicator_color2 Gold
#property indicator_color3 Lime
#property indicator_color4 Gold
#property indicator_color5 Lime
#property indicator_color6 Gold
//---- input parameters
extern double SLDeviation = 1.00; // Offset Factor
extern int    BBPeriod = 8;      // Bollinger Bands Period
extern int    BBDeviation = 1;    // Deviation was 2
extern int    Signal=1;       // Display signals mode: 1-Signals & Stops; 0-only Stops; 2-only Signals;
extern int    Line=1;         // Display line mode: 0-no,1-yes  
extern bool   SoundON = false;

//#include <Debug.mqh>

//---- indicator buffers
double UpTrendBuffer[];   // dots
double DownTrendBuffer[];
double UpTrendSignal[];   // arrows
double DownTrendSignal[]; 
double UpTrendLine[];
double DownTrendLine[];   // lines
double MiddleLine[];
double BBWidth[];

int init() {
//DebugOpen("", "BBS.dbg");
  string short_name;
   
  SetIndexBuffer(0,UpTrendBuffer);
  SetIndexBuffer(1,DownTrendBuffer);
  SetIndexBuffer(2,UpTrendSignal);
  SetIndexBuffer(3,DownTrendSignal);
  SetIndexBuffer(4,UpTrendLine);
  SetIndexBuffer(5,DownTrendLine);    
  SetIndexBuffer(6,MiddleLine);
  SetIndexBuffer(7,BBWidth); 
  
  SetIndexStyle(0,DRAW_ARROW,0,0);
  SetIndexStyle(1,DRAW_ARROW,0,0);
  SetIndexStyle(2,DRAW_ARROW,0,1);
  SetIndexStyle(3,DRAW_ARROW,0,1);
  SetIndexStyle(4,DRAW_LINE);
  SetIndexStyle(5,DRAW_LINE);
   
  SetIndexArrow(0,159);   // dot
  SetIndexArrow(1,159);   // dot
  SetIndexArrow(2,233);   // up
  SetIndexArrow(3,234);   // down
  IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
//---- name for DataWindow and indicator subwindow label
   short_name="BBands Stop("+BBPeriod+","+BBDeviation+")";
  IndicatorShortName(short_name);
  SetIndexLabel(0,"UpTrend Stop");
  SetIndexLabel(1,"DownTrend Stop");
  SetIndexLabel(2,"UpTrend Signal");
  SetIndexLabel(3,"DownTrend Signal");
  SetIndexLabel(4,"UpTrend Line");
  SetIndexLabel(5,"DownTrend Line");
//----
  for (int i=0; i<6; i++) {
    SetIndexDrawBegin(i, BBPeriod);
  }
  return(0);
}

void deinit() {
//DebugClose();
}

//+------------------------------------------------------------------+
//| Bollinger Bands_Stop_v1                                             |
//+------------------------------------------------------------------+
int start() {
  static bool TurnedUp = false;
  static bool TurnedDown = false;  

  int countedBars = IndicatorCounted();
  if (countedBars < 0) return (-1);
  int limit = Bars - countedBars;
  if (limit > 1) limit++;   // recalc [1] candle if new candle is being created only
  
  for (int pos = limit-1; pos >= 0; pos--) {

    double sHi = iBands(NULL, 0, BBPeriod, BBDeviation, 0, PRICE_CLOSE, MODE_UPPER, pos);
	  if (sHi == EMPTY_VALUE) {
	    sHi = 0;
	  }
	  double sLo = iBands(NULL, 0, BBPeriod, BBDeviation, 0, PRICE_CLOSE, MODE_LOWER, pos);
	  if (sLo == EMPTY_VALUE) {
	    sLo = 0;
	  }
    double prevSHi = iBands(NULL, 0, BBPeriod, BBDeviation, 0, PRICE_CLOSE, MODE_UPPER, pos+1);
	  double prevSLo = iBands(NULL, 0, BBPeriod, BBDeviation, 0, PRICE_CLOSE, MODE_LOWER, pos+1);
	  
	  MiddleLine[pos] = (sHi+sLo)/2;	  
	  BBWidth[pos] = sHi-MiddleLine[pos];

    int trend = 0;
    if (UpTrendBuffer[pos+1] != EMPTY_VALUE) {
      trend = 1;
    } else if (DownTrendBuffer[pos+1] != EMPTY_VALUE) {
      trend = -1;
    }
	  
	  if (Close[pos] > prevSHi) {    // close breaks upper band
	    trend = 1;
	  } else if (Close[pos] < prevSLo) {    // ... lower band
	    trend = -1;
    }

    
	  if (trend>0) {
	    if (UpTrendBuffer[pos+1] == EMPTY_VALUE) {
//  	    sLo = MathMax(sLo, prevSLo);  // sticky band from previous period, also protect SL
  	  }
	  } else if (trend<0) {
	    if (DownTrendBuffer[pos+1] == EMPTY_VALUE) {
//  	    sHi = MathMin(sHi, prevSHi);
  	  }
	  }

    double bsW = SLDeviation * BBWidth[pos];  // widden band which goes with trend
	  double bsHi = MiddleLine[pos] + bsW;
	  double bsLo = MiddleLine[pos] - bsW;

//    Debug(pos, sLo, sHi, bsLo, bsHi, bsW, MiddleLine[pos], BBWidth[pos]);
	  
	  if(trend>0) {
	    if (UpTrendBuffer[pos+1] != EMPTY_VALUE) {   // sticky band
//  	    bsLo = MathMax(bsLo, UpTrendBuffer[pos+1]);
  	  }
	  } else if (trend<0) {
	    if (DownTrendBuffer[pos+1] != EMPTY_VALUE) {
//  	    bsHi = MathMin(bsHi, DownTrendBuffer[pos+1]);
  	  }
	  }

    if (pos >= Bars - BBPeriod) {
    
      UpTrendBuffer[pos] = EMPTY_VALUE;
      DownTrendBuffer[pos] = EMPTY_VALUE;
      UpTrendSignal[pos] = EMPTY_VALUE;
      DownTrendSignal[pos] = EMPTY_VALUE;
      UpTrendLine[pos] = EMPTY_VALUE;
      DownTrendLine[pos] = EMPTY_VALUE;
          
	  } else if (trend>0) {
	    if (Signal>0 && UpTrendBuffer[pos+1] == EMPTY_VALUE) {
	      UpTrendSignal[pos] = bsLo;   // arrows
        if (SoundON==true && pos==0 && !TurnedUp) {
          Alert("BBands Stop Alert Buy --> ",Symbol(),"@TF",Period());
          TurnedUp = true;
          TurnedDown = false;
        }
	    } else {
	      UpTrendSignal[pos] = EMPTY_VALUE;
	    }
      UpTrendBuffer[pos] = bsLo;
      if (Line>0) 
        UpTrendLine[pos] = bsLo; 
        
//	    if (Signal==2) 
//	      UpTrendBuffer[pos] = EMPTY_VALUE;
	    DownTrendSignal[pos] = EMPTY_VALUE;
	    DownTrendBuffer[pos] = EMPTY_VALUE;
	    DownTrendLine[pos] = EMPTY_VALUE;
	  } else if (trend<0) {
	    if (Signal>0 && DownTrendBuffer[pos+1] == EMPTY_VALUE) {
	      DownTrendSignal[pos] = bsHi;
        if (SoundON==true && pos==0 && !TurnedDown) {
          Alert("Bbands Stop Alert Sell --> ",Symbol(),"@TF",Period());
          TurnedDown = true;
          TurnedUp = false;
        }
	    } else {
	      DownTrendSignal[pos] = EMPTY_VALUE;   // nothing 
	    }
      DownTrendBuffer[pos] = bsHi;
      if (Line>0)
        DownTrendLine[pos] = bsHi;
       
//	    if (Signal==2) 
//	      DownTrendBuffer[pos] = EMPTY_VALUE;
  	  UpTrendSignal[pos] = EMPTY_VALUE;
	    UpTrendBuffer[pos] = EMPTY_VALUE;
	    UpTrendLine[pos] = EMPTY_VALUE;
	  }	  
  
  }
/*

  original code, programmed in absolutely wrong way !!!
  
  int    i,shift,trend;
  double smax[25000],smin[25000],bsmax[25000],bsmin[25000];
   
  for (shift=Nbars;shift>=0;shift--) {
    UpTrendBuffer[shift]=0;
    DownTrendBuffer[shift]=0;
    UpTrendSignal[shift]=0;
    DownTrendSignal[shift]=0;
    UpTrendLine[shift]=EMPTY_VALUE;
    DownTrendLine[shift]=EMPTY_VALUE;
  }
   
  for (shift=Nbars-Length-1;shift>=0;shift--) {	
    smax[shift]=iBands(NULL,0,Length,Deviation,0,PRICE_CLOSE,MODE_UPPER,shift);
	  smin[shift]=iBands(NULL,0,Length,Deviation,0,PRICE_CLOSE,MODE_LOWER,shift);
	
	  if (Close[shift] > smax[shift+1])    // close breaks upper band
	    trend=1;  
	  if (Close[shift] < smin[shift+1])    // ... lower band
	    trend=-1;
		 	
	  if(trend>0 && smin[shift]<smin[shift+1])   // sticky band from previous period
	    smin[shift]= smin[shift+1];
	  if(trend<0 && smax[shift]>smax[shift+1]) 
	    smax[shift] = smax[shift+1];
	  	  
	  bsmax[shift]=smax[shift]+0.5*(MoneyRisk-1)*(smax[shift]-smin[shift]);  // widden band which goes with trend
	  bsmin[shift]=smin[shift]-0.5*(MoneyRisk-1)*(smax[shift]-smin[shift]);
		
	  if(trend>0 && bsmin[shift]<bsmin[shift+1])   // narrow
	    bsmin[shift]=bsmin[shift+1];
	  if(trend<0 && bsmax[shift]>bsmax[shift+1]) 
	    bsmax[shift]=bsmax[shift+1];
	  
	  if (trend>0) {
	    if (Signal>0 && UpTrendBuffer[shift+1]==-1.0) {
	      UpTrendSignal[shift]=bsmin[shift];   // arrows
	      UpTrendBuffer[shift]=bsmin[shift];
	      if(Line>0) 
	        UpTrendLine[shift]=bsmin[shift]; 
        if (SoundON==true && shift==0 && !TurnedUp) {
          Alert("BBands Stop Alert Buy --> ",Symbol(),"@TF",Period());
          TurnedUp = true;
          TurnedDown = false;
        }
	    } else {
	      UpTrendBuffer[shift]=bsmin[shift];
	      if(Line>0) 
	        UpTrendLine[shift]=bsmin[shift];
	      UpTrendSignal[shift]=-1;
	    }
	    if (Signal==2) 
	      UpTrendBuffer[shift]=0;   
	    DownTrendSignal[shift]=-1;
	    DownTrendBuffer[shift]=-1.0;
	    DownTrendLine[shift]=EMPTY_VALUE;
	  }
	  if (trend<0) {
	    if (Signal>0 && DownTrendBuffer[shift+1]==-1.0) {
	      DownTrendSignal[shift]=bsmax[shift];
	      DownTrendBuffer[shift]=bsmax[shift];
	      if(Line>0) 
	        DownTrendLine[shift]=bsmax[shift];
        if (SoundON==true && shift==0 && !TurnedDown) {
          Alert("Bbands Stop Alert Sell --> ",Symbol(),"@TF",Period());
          TurnedDown = true;
          TurnedUp = false;
        }
	    } else {
	      DownTrendBuffer[shift]=bsmax[shift];
	      if(Line>0)
	        DownTrendLine[shift]=bsmax[shift];
	      DownTrendSignal[shift]=-1;   // nothing 
	    }
	    if (Signal==2) 
	      DownTrendBuffer[shift]=0;    
  	  UpTrendSignal[shift]=-1;
	    UpTrendBuffer[shift]=-1.0;
	    UpTrendLine[shift]=EMPTY_VALUE;
	  }	  
	}
  return(0);	 */
}

